@@ -40,7 +40,11 @@
 #include <errno.h>
 #include <getopt.h>
 #include <fcntl.h>
+#if NCH
+#include <ncurses.h>
+#else
 #include <curses.h>
+#endif
 #include <signal.h>
 #include <math.h>
 #include <string.h>
@@ -48,6 +52,7 @@
 #include <linux/genhd.h>
 #include <linux/hdreg.h>
 #include <linux/fs.h>		/* for BLKRRPART */
+#include "../version.h"
 
 #if defined(__GNUC__) || defined(HAS_LONG_LONG)
 typedef long long ext2_loff_t;
@@ -1990,7 +1995,7 @@ void draw_screen(void)
 	mvaddstr(WARNING_START, 0, line);
 
 
-    sprintf(line, "cfdisk %s", VERSION);
+    sprintf(line, "cfdisk %s/%s", VERSION,util_linux_version);
     mvaddstr(HEADER_START, (COLS-strlen(line))/2, line);
     sprintf(line, "Disk Drive: %s", disk_device);
     mvaddstr(HEADER_START+2, (COLS-strlen(line))/2, line);
