@@ -225,7 +225,7 @@ void menu(void)
 	puts("Command action\n"
 		"   a   toggle a bootable flag\n"
                 "   b   edit bsd disklabel\n" /* bf */
-		"   c   toggle the dos compatiblity flag\n"
+		"   c   toggle the dos compatibility flag\n"
 		"   d   delete a partition\n"
 		"   l   list known partition types\n"
 		"   m   print this menu\n"
@@ -498,7 +498,7 @@ void get_boot(void)
 int read_line(void)
 {
 	if (!fgets(line_buffer, LINE_LENGTH, stdin))
-	        exit(0);
+		return 0;
 	line_ptr = line_buffer;
 	while (*line_ptr && !isgraph(*line_ptr))
 		line_ptr++;
