@@ -6,16 +6,22 @@
  *   modify it under the terms of the gnu general public license.
  *   there is no warranty.
  *
- *   $Author: faith $
- *   $Revision: 1.8 $
- *   $Date: 1995/10/12 14:46:35 $
+ *   $Author: janl $
+ *   $Revision: 1.11 $
+ *   $Date: 1996/11/11 22:26:21 $
  *
  * Updated Thu Oct 12 09:19:26 1995 by faith@cs.unc.edu with security
  * patches from Zefram <A.Main@dcs.warwick.ac.uk>
  *
+ *  Hacked by Peter Breitenlohner, peb@mppmu.mpg.de,
+ *    to allow peaceful coexistence with yp: using the changes by
+ *     Alvaro Martinez Echevarria, alvaro@enano.etsit.upm.es, from
+ *     passwd.c (now moved to setpwnam.c);
+ *    to remove trailing empty fields.  Oct 5, 96.
+ *
  */
 
-static char rcsId[] = "$Version: $Id: chfn.c,v 1.8 1995/10/12 14:46:35 faith Exp $ $";
+static char rcsId[] = "$Version: $Id: chfn.c,v 1.11 1996/11/11 22:26:21 janl Exp $ $";
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -26,6 +32,9 @@ static char rcsId[] = "$Version: $Id: chfn.c,v 1.8 1995/10/12 14:46:35 faith Exp
 #include <errno.h>
 #include <ctype.h>
 #include <getopt.h>
+#include "../version.h"
+
+extern int is_local(char *);
 
 #undef P
 #if __STDC__
@@ -38,7 +47,6 @@ typedef unsigned char boolean;
 #define false 0
 #define true 1
 
-static char *version_string = "chfn 0.9a beta";
 static char *whoami;
 
 static char buf[1024];
@@ -114,7 +122,13 @@ int main (argc, argv)
 	    return (-1); }
     }
 
-    /* reality check */
+    if (!(is_local(oldf.username))) {
+       fprintf (stderr, "%s: can only change local entries; use yp%s instead.\n",
+           whoami, whoami);
+       exit(1);
+    }
+
+    /* Reality check */
     if (uid != 0 && uid != oldf.pw->pw_uid) {
 	errno = EACCES;
 	perror (whoami);
@@ -176,7 +190,7 @@ static boolean parse_argv (argc, argv, pinfo)
 	if (c == EOF) break;
 	/* version?  output version and exit. */
 	if (c == 'v') {
-	    printf ("%s\n", version_string);
+	    printf ("%s\n", util_linux_version);
 	    exit (0);
 	}
 	if (c == 'u') {
@@ -404,6 +418,12 @@ static int save_new_data (pinfo)
     sprintf (gecos, "%s,%s,%s,%s,%s", pinfo->full_name, pinfo->office,
 	     pinfo->office_phone, pinfo->home_phone, pinfo->other);
 
+    /* remove trailing empty fields (but not subfields of pinfo->other) */
+    if (! pinfo->other[0] ) {
+       while (len > 0 && gecos[len-1] == ',') len--;
+       gecos[len] = 0;
+    }
+
     /* write the new struct passwd to the passwd file. */
     pinfo->pw->pw_gecos = gecos;
     if (setpwnam (pinfo->pw) < 0) {
