@@ -6,16 +6,25 @@
  *   modify it under the terms of the gnu general public license.
  *   there is no warranty.
  *
- *   $Author: faith $
- *   $Revision: 1.8 $
- *   $Date: 1995/10/12 14:46:35 $
+ *   $Author: janl $
+ *   $Revision: 1.14 $
+ *   $Date: 1996/11/11 22:26:21 $
  *
  * Updated Thu Oct 12 09:33:15 1995 by faith@cs.unc.edu with security
- * patches from Zefram <A.Main@dcs.warwick.ac.uk>
+ *   patches from Zefram <A.Main@dcs.warwick.ac.uk>
+ *
+ * Updated Mon Jul  1 18:46:22 1996 by janl@math.uio.no with security
+ *   suggestion from Zefram.  Disallowing users with shells not in /etc/shells
+ *   from changing their shell.
+ *
+ *  Hacked by Peter Breitenlohner, peb@mppmu.mpg.de,
+ *    to allow peaceful coexistence with yp: using the changes by
+ *     Alvaro Martinez Echevarria, alvaro@enano.etsit.upm.es, from
+ *     passwd.c (now moved to setpwnam.c). Oct 5, 96.
  *
  */
 
-static char rcsId[] = "$Version: $Id: chsh.c,v 1.8 1995/10/12 14:46:35 faith Exp $ $";
+static char rcsId[] = "$Version: $Id: chsh.c,v 1.14 1996/11/11 22:26:21 janl Exp $ $";
 
 #if 0
 #define _POSIX_SOURCE 1
@@ -30,6 +39,9 @@ static char rcsId[] = "$Version: $Id: chsh.c,v 1.8 1995/10/12 14:46:35 faith Exp
 #include <errno.h>
 #include <ctype.h>
 #include <getopt.h>
+#include "../version.h"
+
+extern int is_local(char *);
 
 #undef P
 #if __STDC__
@@ -42,7 +54,10 @@ typedef unsigned char boolean;
 #define false 0
 #define true 1
 
-static char *version_string = "chsh 0.9a beta";
+/* Only root is allowed to assign a luser a non-listed shell, by default */
+#define ONLY_LISTED_SHELLS 1
+
+
 static char *whoami;
 
 static char buf[FILENAME_MAX];
@@ -96,21 +111,21 @@ int main (argc, argv)
 	    return (-1); }
     }
 
+    if (!(is_local(pw->pw_name))) {
+       fprintf (stderr, "%s: can only change local entries; use yp%s instead.\n
+",
+           whoami, whoami);
+       exit(1);
+    }
+
     oldshell = pw->pw_shell;
     if (!oldshell[0]) oldshell = "/bin/sh";
 
     /* reality check */
-#if 0
-				/* Require current shell to be in list.
-                                   This is not a reasonable expectation on
-                                   most Linux systems, and the error is
-                                   confusing.  */
     if (uid != 0 && (uid != pw->pw_uid || !get_shell_list(oldshell))) {
-#else
-    if (uid != 0 && uid != pw->pw_uid) {
-#endif
 	errno = EACCES;
-	perror (whoami);
+	fprintf(stderr,"%s: Your shell is not in /etc/shells, shell change"
+		" denied\n",whoami);
 	return (-1);
     }
     
@@ -178,7 +193,7 @@ static void parse_argv (argc, argv, pinfo)
 	case EOF:
 	    break;
 	case 'v':
-	    printf ("%s\n", version_string);
+	    printf ("%s\n", util_linux_version);
 	    exit (0);
 	case 'u':
 	    usage (stdout);
@@ -246,7 +261,7 @@ static char *prompt (question, def_val)
     if (len <= 0) return NULL;
     ans[len] = 0;
     cp = (char *) xmalloc (len + 1);
-    strcpy (cp, buf);
+    strcpy (cp, ans);
     return cp;
 }
 
@@ -287,17 +302,17 @@ static int check_shell (shell)
 #if ONLY_LISTED_SHELLS
     if (! get_shell_list (shell)) {
        if (!getuid())
-	  printf ("Warning: \"%s\" is not listed as a valid shell.\n", shell);
+	  printf ("Warning: \"%s\" is not listed in /etc/shells\n", shell);
        else {
-	  printf ("%s: \"%s\" is not listed as a valid shell.\n",
+	  printf ("%s: \"%s\" is not listed in /etc/shells.\n",
 		  whoami, shell);
-	  printf( "%s: use -l option to see list\n" );
+	  printf( "%s: use -l option to see list\n", whoami );
 	  exit(1);
        }
     }
 #else
     if (! get_shell_list (shell)) {
-       printf ("Warning: \"%s\" is not listed as a valid shell.\n", shell);
+       printf ("Warning: \"%s\" is not listed in /etc/shells.\n", shell);
        printf( "Use %s -l to see list.\n", whoami );
     }
 #endif
