@@ -128,7 +128,7 @@ static char sccsid[] = "@(#)login.c	5.40 (Berkeley) 5/9/89";
 #include <shadow.h>
 #endif
 
-#ifndef linux
+#ifndef __linux__
 #include <tzfile.h>
 #endif
 #include <lastlog.h>
@@ -178,7 +178,7 @@ int	kerror = KSUCCESS, notickets = 1;
  * This bounds the time given to login.  Not a define so it can
  * be patched on machines where it's too small.
  */
-#ifndef linux
+#ifndef __linux__
 int	timeout = 300;
 #else
 int     timeout = 60;
@@ -190,7 +190,7 @@ char	term[64], *hostname, *username, *tty;
 struct  hostent hostaddress;
 char	thishost[100];
 
-#ifndef linux
+#ifndef __linux__
 struct	sgttyb sgttyb;
 struct	tchars tc = {
 	CINTR, CQUIT, CSTART, CSTOP, CEOT, CBRK
@@ -240,7 +240,7 @@ main(argc, argv)
 	void timedout();
 	char *termenv;
 
-#ifdef linux
+#ifdef __linux__
 	char tmp[100];
 	/* Just as arbitrary as mountain time: */
         /* (void)setenv("TZ", "MET-1DST",0); */
@@ -313,7 +313,7 @@ main(argc, argv)
 	} else
 		ask = 1;
 
-#ifndef linux
+#ifndef __linux__
 	ioctlval = 0;
 	(void)ioctl(0, TIOCLSET, &ioctlval);
 	(void)ioctl(0, TIOCNXCL, 0);
@@ -376,7 +376,7 @@ main(argc, argv)
 
 	for (cnt = 0;; ask = 1) {
 		ioctlval = 0;
-#ifndef linux
+#ifndef __linux__
 		(void)ioctl(0, TIOCSETD, &ioctlval);
 #endif
 
@@ -404,7 +404,10 @@ main(argc, argv)
 		else
 			salt = "xx";
 
-		checktty(username, tty, pwd); /* in checktty.c */
+		if (pwd) {
+			initgroups(username, pwd->pw_gid);
+			checktty(username, tty, pwd); /* in checktty.c */
+		}
 
 		/* if user not super-user, check for disabled logins */
 		if (pwd == NULL || pwd->pw_uid)
@@ -483,8 +486,8 @@ main(argc, argv)
 			break;
 
 		(void)printf("Login incorrect\n");
-		failures++;
 		badlogin(username); /* log ALL bad logins */
+		failures++;
 
 		/* we allow 10 tries, but after 3 we start backing off */
 		if (++cnt > 3) {
@@ -543,7 +546,7 @@ main(argc, argv)
 	    setregid(-1, egid);
 	}
 
-#ifndef linux
+#ifndef __linux__
 #ifdef KERBEROS
 	if (notickets && !quietlog)
 		(void)printf("Warning: no Kerberos tickets issued\n");
@@ -635,7 +638,7 @@ main(argc, argv)
 
 	dolastlog(quietlog);
 	
-#ifndef linux
+#ifndef __linux__
 	if (!hflag) {					/* XXX */
 		static struct winsize win = { 0, 0, 0, 0 };
 
@@ -652,7 +655,6 @@ main(argc, argv)
 #endif
 
 	setgid(pwd->pw_gid);
-	initgroups(username, pwd->pw_gid);
 
 #ifdef HAVE_QUOTA
 	quota(Q_DOWARN, pwd->pw_uid, (dev_t)-1, 0);
@@ -660,7 +662,7 @@ main(argc, argv)
 
 	if (*pwd->pw_shell == '\0')
 		pwd->pw_shell = _PATH_BSHELL;
-#ifndef linux
+#ifndef __linux__
 	/* turn on new line discipline for the csh */
 	else if (!strcmp(pwd->pw_shell, _PATH_CSHELL)) {
 		ioctlval = NTTYDISC;
@@ -683,7 +685,7 @@ main(argc, argv)
 	  memset(environ, 0, sizeof(char*));
 	}
 
-#ifndef linux
+#ifndef __linux__
 	(void)setenv("HOME", pwd->pw_dir, 1);
 	(void)setenv("SHELL", pwd->pw_shell, 1);
 	if (term[0] == '\0')
@@ -711,14 +713,25 @@ main(argc, argv)
 #endif
 
 	if (tty[sizeof("tty")-1] == 'S')
-		syslog(LOG_INFO, "DIALUP %s, %s", tty, pwd->pw_name);
+	    syslog(LOG_INFO, "DIALUP AT %s BY %s", tty, pwd->pw_name);
 
-	if (pwd->pw_uid == 0)
-		if (hostname)
-			syslog(LOG_NOTICE, "ROOT LOGIN ON %s FROM %s",
-			    tty, hostname);
-		else
-			syslog(LOG_NOTICE, "ROOT LOGIN ON %s", tty);
+	/* allow tracking of good logins.
+	   -steve philp (sphilp@mail.alliance.net) */
+
+	if (pwd->pw_uid == 0) {
+	    if (hostname)
+		syslog(LOG_NOTICE, "ROOT LOGIN ON %s FROM %s",
+		       tty, hostname);
+	    else
+		syslog(LOG_NOTICE, "ROOT LOGIN ON %s", tty);
+	} else {
+	    if (hostname) 
+		syslog(LOG_INFO, "LOGIN ON %s BY %s FROM %s", tty, 
+		       pwd->pw_name, hostname);
+	    else 
+		syslog(LOG_INFO, "LOGIN ON %s BY %s", tty, 
+		       pwd->pw_name);
+	}
 
 	if (!quietlog) {
 		struct stat st;
@@ -837,7 +850,7 @@ void timedout()
 int
 rootterm(ttyn)
 	char *ttyn;
-#ifndef linux
+#ifndef __linux__
 {
 	struct ttyent *t;
 
@@ -959,7 +972,7 @@ badlogin(name)
 #undef	UNKNOWN
 #define	UNKNOWN	"su"
 
-#ifndef linux
+#ifndef __linux__
 char *
 stypeof(ttyid)
 	char *ttyid;
