@@ -221,7 +221,7 @@ main(argc, argv)
 	}
 
 	
-	if((fd = open(_PATH_NOLOGIN, O_WRONLY|O_CREAT)) >= 0) {
+	if((fd = open(_PATH_NOLOGIN, O_WRONLY|O_CREAT, 0644)) >= 0) {
 		WR("\r\nThe system is being shut down within 5 minutes\r\n");
 		write(fd, message, strlen(message));
 		WR("\r\nLogin is therefore prohibited.\r\n");
@@ -249,13 +249,13 @@ main(argc, argv)
 	closelog();
 
 	if(opt_fast)
-		if((fd = open("/fastboot", O_WRONLY|O_CREAT)) >= 0)
+		if((fd = open("/fastboot", O_WRONLY|O_CREAT, 0644)) >= 0)
 			close(fd);
 
 	kill(1, SIGTSTP);	/* tell init not to spawn more getty's */
 	write_wtmp();
 	if(opt_single)
-		close(open(_PATH_SINGLE, O_CREAT|O_WRONLY));
+		close(open(_PATH_SINGLE, O_CREAT|O_WRONLY, 0644));
 		
 	sync();
 
@@ -288,9 +288,12 @@ main(argc, argv)
 	if(opt_reboot) {
 		reboot(0xfee1dead, 672274793, 0x1234567);
 	} else {
+	        freopen(_PATH_CONSOLE, "w", stdout);
 		printf("\nNow you can turn off the power...\n");
-		/* allow C-A-D now, faith@cs.unc.edu */
+		fflush(stdout);
+		/* allow C-A-D now, faith@cs.unc.edu, re-fixed 8-Jul-96 */
 		reboot(0xfee1dead, 672274793, 0x89abcdef);
+		reboot(0xfee1dead, 672274793, 0xcdef0123);
 	}
 	/* NOTREACHED */
 	exit(0); /* to quiet gcc */
@@ -364,7 +367,7 @@ write_wtmp()
 	time(&ut.ut_time);
 	ut.ut_type = BOOT_TIME;
 	
-	if((fd = open(_PATH_WTMP, O_WRONLY|O_APPEND)) >= 0) {
+	if((fd = open(_PATH_WTMP, O_WRONLY|O_APPEND, 0644)) >= 0) {
 		write(fd, (char *)&ut, sizeof(ut));
 		close(fd);
 	}
