@@ -59,7 +59,7 @@ char *sys_signame[NSIG] = {
 #endif
 
 int main (int argc, char *argv[]);
-char *mybasename(char *pathname);
+extern char *mybasename(char *);
 int signame_to_signum (char *sig);
 int arg_to_signum (char *arg);
 void nosig (char *name);
@@ -186,13 +186,6 @@ int main (int argc, char *argv[])
     return (errors);
 }
 
-char *mybasename (char *path)
-{
-    char *cp;
-
-    cp = strrchr (path, '/');
-    return (cp ? cp + 1 : path);
-}
 
 int signame_to_signum (char *sig)
 {
