@@ -8,6 +8,9 @@
  *
  * Enhancements by Mika Liljeberg (liljeber@cs.Helsinki.FI)
  *
+ * Beep modifications by Christophe Jolif (cjolif@storm.gatelink.fr.net)
+ *
+ * Sanity increases by Cafeine Addict [sic].
  *
  * Syntax:
  *
@@ -48,6 +51,8 @@
  *   [ -msg [on|off] ]
  *   [ -msglevel [0-8] ]
  *   [ -powersave [on|off] ]
+ *   [ -blength [0-2000] ]
+ *   [ -bfreq freq ]
  *
  *
  * Semantics:
@@ -79,18 +84,24 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 #include <ctype.h>
 #include <termcap.h>
-#include <linux/config.h>
 #include <sys/time.h>
 #include <unistd.h>
 #include <termios.h>
 #include <string.h>
 
-/* for syslog system call */
-#include <linux/unistd.h>
-#include <errno.h>
-_syscall3(int, syslog, int, type, char*, buf, int, len);
+#ifndef TCGETS
+/* TCGETS is either defined in termios.h, or here: */
+#include <asm/ioctls.h>
+#endif
+
+#if __GNU_LIBRARY__ < 5
+# include <linux/unistd.h>
+#define __NR_klogctl __NR_syslog
+_syscall3(int, klogctl, int, type, char*, buf, int, len);
+#endif
 
 /* Constants. */
 
@@ -137,6 +148,7 @@ int opt_store, opt_clear, opt_blank, opt_snap, opt_snapfile, opt_standout;
 int opt_append, opt_ulcolor, opt_hbcolor, opt_halfbright, opt_repeat;
 int opt_tabs, opt_clrtabs, opt_regtabs, opt_appcursorkeys, opt_inversescreen;
 int opt_msg, opt_msglevel, opt_powersave;
+int opt_blength, opt_bfreq;
 
 /* Option controls.  The variable names have been contracted to ensure
  * uniqueness.
@@ -150,6 +162,8 @@ int opt_fo_color, opt_ba_color;	/* Colors. */
 int opt_ul_color, opt_hb_color;
 int opt_cl_all;			/* Clear all or rest. */
 int opt_bl_min;			/* Blank screen. */
+int opt_blength_l;
+int opt_bfreq_f;
 int opt_sn_num = 0;		/* Snap screen. */
 int opt_st_attr;
 int opt_rt_len;			/* regular tab length */
@@ -253,30 +267,31 @@ int *bad_arg;			/* Set to true if an error is detected. */
   if (argc != 1 || *option) *bad_arg = TRUE;
   *option = TRUE;
   if (argc == 1) {
-	if (strcmp(argv[0], "black") == 0)
-		*opt_color = BLACK;
-	else if (strcmp(argv[0], "red") == 0)
-		*opt_color = RED;
-	else if (strcmp(argv[0], "green") == 0)
-		*opt_color = GREEN;
-	else if (strcmp(argv[0], "yellow") == 0)
-		*opt_color = YELLOW;
-	else if (strcmp(argv[0], "blue") == 0)
-		*opt_color = BLUE;
-	else if (strcmp(argv[0], "magenta") == 0)
-		*opt_color = MAGENTA;
-	else if (strcmp(argv[0], "cyan") == 0)
-		*opt_color = CYAN;
-	else if (strcmp(argv[0], "white") == 0)
-		*opt_color = WHITE;
-	else if (strcmp(argv[0], "default") == 0)
-		*opt_color = DEFAULT;
-	else if (isdigit(argv[0][0]))
-	    *opt_color = atoi(argv[0]);
-	else    
-		*bad_arg = TRUE;
-       if(*opt_color < 0 || *opt_color > 15)
-	    *bad_arg = TRUE;
+    if (strcmp(argv[0], "black") == 0)
+      *opt_color = BLACK;
+    else if (strcmp(argv[0], "red") == 0)
+      *opt_color = RED;
+    else if (strcmp(argv[0], "green") == 0)
+      *opt_color = GREEN;
+    else if (strcmp(argv[0], "yellow") == 0)
+      *opt_color = YELLOW;
+    else if (strcmp(argv[0], "blue") == 0)
+      *opt_color = BLUE;
+    else if (strcmp(argv[0], "magenta") == 0)
+      *opt_color = MAGENTA;
+    else if (strcmp(argv[0], "cyan") == 0)
+      *opt_color = CYAN;
+    else if (strcmp(argv[0], "white") == 0)
+      *opt_color = WHITE;
+    else if (strcmp(argv[0], "default") == 0)
+      *opt_color = DEFAULT;
+    else if (isdigit(argv[0][0]))
+      *opt_color = atoi(argv[0]);
+    else    
+      *bad_arg = TRUE;
+    
+    if(*opt_color < 0 || *opt_color > 7)
+      *bad_arg = TRUE;
   }
 }
 
@@ -514,6 +529,46 @@ int *bad_arg;			/* Set to true if an error is detected. */
   }
 }
 
+
+void parse_blength(argc, argv, option, opt_all, bad_arg)
+int argc;			/* Number of arguments for this option. */
+char *argv[];			/* Arguments for this option. */
+int *option;			/* Clear flag to set. */
+int *opt_all;			
+int *bad_arg;			/* Set to true if an error is detected. */
+{
+/* Parse  -blength specification. */
+
+  if (argc > 1 || *option) *bad_arg = TRUE;
+  *option = TRUE;
+  if (argc == 1) {
+	*opt_all = atoi(argv[0]);
+	if (*opt_all > 2000)
+		*bad_arg = TRUE;
+  } else {
+	*opt_all = 0;
+  }
+}
+
+void parse_bfreq(argc, argv, option, opt_all, bad_arg)
+int argc;			/* Number of arguments for this option. */
+char *argv[];			/* Arguments for this option. */
+int *option;			/* Clear flag to set. */
+int *opt_all;			
+int *bad_arg;			/* Set to true if an error is detected. */
+{
+/* Parse  -bfreq specification. */
+
+  if (argc > 1 || *option) *bad_arg = TRUE;
+  *option = TRUE;
+  if (argc == 1) {
+	*opt_all = atoi(argv[0]);
+  } else {
+	*opt_all = 0;
+  }
+}
+
+
 void show_tabs()
 {
   int i, co = tgetnum("co");
@@ -613,6 +668,10 @@ int *bad_arg;			/* Set to true if an error is detected. */
 	parse_msglevel(argc, argv, &opt_msglevel, &opt_msglevel_num, bad_arg);
   else if (STRCMP(option, "powersave") == 0)
 	parse_switch(argc, argv, &opt_powersave, &opt_ps_on, bad_arg);
+  else if (STRCMP(option, "blength") == 0)
+	parse_blength(argc, argv, &opt_blength, &opt_blength_l, bad_arg);
+  else if (STRCMP(option, "bfreq") == 0)
+   	parse_bfreq(argc, argv, &opt_bfreq, &opt_bfreq_f, bad_arg);
 #if 0
   else if (STRCMP(option, "standout") == 0)
 	parse_standout(argc, argv, &opt_standout, &opt_st_attr, bad_arg);
@@ -677,11 +736,12 @@ char *prog_name;		/* Name of this program. */
   fprintf(stderr, "  [ -msg [on|off] ]\n");
   fprintf(stderr, "  [ -msglevel [0-8] ]\n");
   fprintf(stderr, "  [ -powersave [on|off] ]\n");
+  fprintf(stderr, "  [ -blength [0-2000] ]\n");
+  fprintf(stderr, "  [ -bfreq freqnumber ]\n");
 }
 
 char tc_ent_buf[TC_ENT_SIZE];	/* Buffer for storing a termcap entry. */
 
-/* Modified to work with ncurses termcap emulation - mrn 22 May 96 */
 char *tc_entry(name)
 char *name;			/* Termcap capability string to lookup. */
 {
@@ -690,15 +750,10 @@ char *name;			/* Termcap capability string to lookup. */
  */
 
   char *buf_ptr;
-  char *result;
 
   buf_ptr = tc_ent_buf;
-  result = tgetstr(name, &buf_ptr);
-  if (result == NULL) {
-    tc_ent_buf[0] = '\0';
-    result = tc_ent_buf;
-  }
-  return result;
+  if (tgetstr(name, &buf_ptr) == NULL) tc_ent_buf[0] = '\0';
+  return tc_ent_buf;
 }
 
 void perform_sequence(vcterm)
@@ -932,7 +987,7 @@ int vcterm;			/* Set if terminal is a virtual console. */
     if (opt_tb_array[0] == -1)
       printf("\033[3g");
     else
-      for(i=0; opt_tb_array[i]; i++)
+      for(i=0; opt_tb_array[i] > 0; i++)
         printf("\033[%dG\033[g", opt_tb_array[i]);
     putchar('\r');
   }
@@ -948,7 +1003,7 @@ int vcterm;			/* Set if terminal is a virtual console. */
   }
 
   /* -blank [0-60]. */
-  if (opt_blank) 
+  if (opt_blank && vcterm) 
     printf("\033[9;%d]", opt_bl_min);
     
   /* -powersave [on|off] (console) */
@@ -985,22 +1040,33 @@ int vcterm;			/* Set if terminal is a virtual console. */
   if (opt_msg && vcterm) {
        if (opt_msg_on)
                /* 7 -- Enable printk's to console */
-               result = syslog(7, NULL, 0);
+               result = klogctl(7, NULL, 0);
        else
                /*  6 -- Disable printk's to console */
-               result = syslog(6, NULL, 0);
+               result = klogctl(6, NULL, 0);
 
        if (result != 0)
-               printf("syslog error: %s\n", strerror(result));
+               printf("klogctl error: %s\n", strerror(result));
   }
 
   /* -msglevel [0-8] */
   if (opt_msglevel && vcterm) {
        /* 8 -- Set level of messages printed to console */
-       result = syslog(8, NULL, opt_msglevel_num);
+       result = klogctl(8, NULL, opt_msglevel_num);
        if (result != 0)
-               printf("syslog error: %s\n", strerror(result));
+               printf("klogctl error: %s\n", strerror(result));
    }
+
+  /* -blength [0-2000] */
+  if (opt_blength && vcterm) {
+       printf("\033[11;%d]", opt_blength_l);
+  }
+  
+  /* -bfreq freqnumber */
+  if (opt_bfreq && vcterm) {
+   	printf("\033[10;%d]", opt_bfreq_f);
+  }
+
 }
 
 extern char *malloc();
