@@ -44,6 +44,7 @@ static char sccsid[] = "@(#)whereis.c	5.5 (Berkeley) 4/18/91";
 #include <sys/param.h>
 #include <sys/dir.h>
 #include <stdio.h>
+#include <string.h>
 #include <ctype.h>
 
 static char *bindirs[] = {
@@ -66,6 +67,8 @@ static char *bindirs[] = {
    "/usr/lib/emacs/19.28/etc",
    "/usr/lib/emacs/19.29/etc",
    "/usr/lib/emacs/19.30/etc",
+   "/usr/lib/emacs/19.31/etc",
+   "/usr/lib/emacs/19.32/etc",
    "/usr/TeX/bin",
    "/usr/tex/bin",
    "/usr/interviews/bin/LINUX",
