@@ -36,7 +36,10 @@
  * Modified for Linux, Mon Mar  8 18:16:24 1993, faith@cs.unc.edu
  * Wed Jun 22 21:41:56 1994, faith@cs.unc.edu:
  *      Added fix from Mike Grupenhoff (kashmir@umiacs.umd.edu)
- *
+ * Mon Jul  1 17:01:39 MET DST 1996, janl@math.uio.no:
+ *      - Added fix from David.Chapell@mail.trincoll.edu enabeling daemons
+ *	  to use write.
+ *      - ANSIed it since I was working on it anyway.
  */
 
 #ifndef lint
@@ -65,11 +68,13 @@ static char sccsid[] = "@(#)write.c	8.1 (Berkeley) 6/6/93";
 #include <locale.h>
 #endif
 
+void search_utmp(char *, char *, char *, uid_t);
+void do_write(char *, char *, uid_t);
+void wr_fputs(char *);
+
 extern int errno;
 
-main(argc, argv)
-	int argc;
-	char **argv;
+int main(int argc, char **argv)
 {
 	register char *cp;
 	time_t atime;
@@ -90,21 +95,25 @@ main(argc, argv)
 	else if (isatty(fileno(stderr)))
 		myttyfd = fileno(stderr);
 	else {
-		(void)fprintf(stderr, "write: can't find your tty\n");
-		exit(1);
+	  	myttyfd = -1;
 	}
-	if (!(mytty = ttyname(myttyfd))) {
+	if (myttyfd != -1) {
+	  if (!(mytty = ttyname(myttyfd))) {
 		(void)fprintf(stderr, "write: can't find your tty's name\n");
 		exit(1);
-	}
-	if (cp = rindex(mytty, '/'))
+	  }
+	  if (cp = rindex(mytty, '/'))
 		mytty = cp + 1;
-	if (term_chk(mytty, &msgsok, &atime, 1))
+	  if (term_chk(mytty, &msgsok, &atime, 1))
 		exit(1);
-	if (!msgsok) {
+	  if (!msgsok) {
 		(void)fprintf(stderr,
 		    "write: you have write permission turned off.\n");
 		exit(1);
+	  }
+	
+	} else {
+	    mytty = "<no tty>";
 	}
 
 	myuid = getuid();
@@ -142,12 +151,13 @@ main(argc, argv)
 	/* NOTREACHED */
 }
 
+
 /*
  * utmp_chk - checks that the given user is actually logged in on
  *     the given tty
  */
-utmp_chk(user, tty)
-	char *user, *tty;
+int utmp_chk(char *user, char *tty)
+
 {
 	struct utmp u;
 	int ufd;
@@ -177,9 +187,8 @@ utmp_chk(user, tty)
  * Special case for writing to yourself - ignore the terminal you're
  * writing from, unless that's the only terminal with messages enabled.
  */
-search_utmp(user, tty, mytty, myuid)
-	char *user, *tty, *mytty;
-	uid_t myuid;
+void search_utmp(char *user, char *tty, char *mytty, uid_t myuid)
+
 {
 	struct utmp u;
 	time_t bestatime, atime;
@@ -251,10 +260,8 @@ search_utmp(user, tty, mytty, myuid)
  * term_chk - check that a terminal exists, and get the message bit
  *     and the access time
  */
-term_chk(tty, msgsokP, atimeP, showerror)
-	char *tty;
-	int *msgsokP, showerror;
-	time_t *atimeP;
+int term_chk(char *tty, int *msgsokP, time_t *atimeP, int showerror)
+
 {
 	struct stat s;
 	char path[MAXPATHLEN];
@@ -274,9 +281,8 @@ term_chk(tty, msgsokP, atimeP, showerror)
 /*
  * do_write - actually make the connection
  */
-do_write(tty, mytty, myuid)
-	char *tty, *mytty;
-	uid_t myuid;
+void do_write(char *tty, char *mytty, uid_t myuid)
+
 {
 	register char *login, *nows;
 	register struct passwd *pwd;
@@ -316,8 +322,7 @@ do_write(tty, mytty, myuid)
 /*
  * done - cleanup and exit
  */
-void
-done()
+void done(void)
 {
 	(void)printf("EOF\r\n");
 	exit(0);
@@ -327,10 +332,10 @@ done()
  * wr_fputs - like fputs(), but makes control characters visible and
  *     turns \n into \r\n
  */
-wr_fputs(s)
-	register char *s;
+void wr_fputs(char *s)
+
 {
-	register char c;
+	char c;
 
 #define	PUTC(c)	if (putchar(c) == EOF) goto err;
 
