@@ -22,24 +22,30 @@
  * Wed Feb  8 12:27:00 1995: Andries.Brouwer@cwi.nl fixed up error messages.
  * Sat Jun  3 20:44:38 1995: Patches from Andries.Brouwer@cwi.nl applied.
  * Tue Sep 26 22:38:20 1995: aeb@cwi.nl, many changes
+ * Fri Feb 23 13:47:00 1996: aeb@cwi.nl, loop device related changes
  *
+ * Fri Apr  5 01:13:33 1996: quinlan@bucknell.edu, fixed up iso9660 autodetect
  */
 
-#include "sundries.h"
 
 #include <linux/fs.h>
 #include <linux/minix_fs.h>
 #include <linux/ext_fs.h>
 #include <linux/ext2_fs.h>
 #include <linux/xia_fs.h>
+#include <linux/iso_fs.h>
+#include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <ctype.h>
+
+#include "sundries.h"
+#include "lomount.h"
+#include "loop.h"
 
 #define PROC_FILESYSTEMS	"/proc/filesystems"
 #define SIZE(a) (sizeof(a)/sizeof(a[0]))
 
-int del_loop (const char *);
-
 /* True for fake mount (-f).  */
 int fake = 0;
 
@@ -65,16 +71,20 @@ int suid = 0;
 struct opt_map
 {
   const char *opt;		/* option name */
+  int  skip;			/* skip in mtab option string */
   int  inv;			/* true if flag value should be inverted */
   int  mask;			/* flag mask value */
 };
 
 /* Custom mount options for our own purposes.  */
+/* We can use the high-order 16 bits, since the mount call
+   has MS_MGC_VAL there. */
 #define MS_NOAUTO	0x80000000
 #define MS_USER		0x40000000
+#define MS_LOOP		0x00010000
 
 /* Options that we keep the mount system call from seeing.  */
-#define MS_NOSYS	(MS_NOAUTO|MS_USER)
+#define MS_NOSYS	(MS_NOAUTO|MS_USER|MS_LOOP)
 
 /* Options that we keep from appearing in the options field in the mtab.  */
 #define MS_NOMTAB	(MS_REMOUNT|MS_NOAUTO|MS_USER)
@@ -82,36 +92,77 @@ struct opt_map
 /* OPTIONS that we make ordinary users have by default.  */
 #define MS_SECURE	(MS_NOEXEC|MS_NOSUID|MS_NODEV)
 
-const struct opt_map opt_map[] =
-{
-  { "defaults",	0, 0		},	/* default options */
-  { "ro",	0, MS_RDONLY	},	/* read-only */
-  { "rw",	1, MS_RDONLY	},	/* read-write */
-  { "exec",	1, MS_NOEXEC	},	/* permit execution of binaries */
-  { "noexec",	0, MS_NOEXEC	},	/* don't execute binaries */
-  { "suid",	1, MS_NOSUID	},	/* honor suid executables */
-  { "nosuid",	0, MS_NOSUID	},	/* don't honor suid executables */
-  { "dev",	1, MS_NODEV	},	/* interpret device files  */
-  { "nodev",	0, MS_NODEV	},	/* don't interpret devices */
-  { "sync",	0, MS_SYNCHRONOUS},	/* synchronous I/O */
-  { "async",	1, MS_SYNCHRONOUS},	/* asynchronous I/O */
-  { "remount",  0, MS_REMOUNT   },      /* Alter flags of mounted FS */
-  { "auto",	1, MS_NOAUTO	},	/* Can be mounted using -a */
-  { "noauto",	0, MS_NOAUTO	},	/* Can  only be mounted explicitly */
-  { "user",	0, MS_USER	},	/* Allow ordinary user to mount */
-  { "nouser",	1, MS_USER	},	/* Forbid ordinary user to mount */
+const struct opt_map opt_map[] = {
+  { "defaults",	0, 0, 0		},	/* default options */
+  { "ro",	1, 0, MS_RDONLY	},	/* read-only */
+  { "rw",	1, 1, MS_RDONLY	},	/* read-write */
+  { "exec",	0, 1, MS_NOEXEC	},	/* permit execution of binaries */
+  { "noexec",	0, 0, MS_NOEXEC	},	/* don't execute binaries */
+  { "suid",	0, 1, MS_NOSUID	},	/* honor suid executables */
+  { "nosuid",	0, 0, MS_NOSUID	},	/* don't honor suid executables */
+  { "dev",	0, 1, MS_NODEV	},	/* interpret device files  */
+  { "nodev",	0, 0, MS_NODEV	},	/* don't interpret devices */
+  { "sync",	0, 0, MS_SYNCHRONOUS},	/* synchronous I/O */
+  { "async",	0, 1, MS_SYNCHRONOUS},	/* asynchronous I/O */
+  { "remount",  0, 0, MS_REMOUNT},      /* Alter flags of mounted FS */
+  { "auto",	0, 1, MS_NOAUTO	},	/* Can be mounted using -a */
+  { "noauto",	0, 0, MS_NOAUTO	},	/* Can  only be mounted explicitly */
+  { "user",	0, 0, MS_USER	},	/* Allow ordinary user to mount */
+  { "nouser",	0, 1, MS_USER	},	/* Forbid ordinary user to mount */
   /* add new options here */
 #ifdef MS_NOSUB
-  { "sub",	1, MS_NOSUB	},	/* allow submounts */
-  { "nosub",	0, MS_NOSUB	},	/* don't allow submounts */
+  { "sub",	0, 1, MS_NOSUB	},	/* allow submounts */
+  { "nosub",	0, 0, MS_NOSUB	},	/* don't allow submounts */
 #endif
-#ifdef  MS_SILENT
-  { "quiet",    0, MS_SILENT    },      /* be quiet  */
-  { "loud",     1, MS_SILENT    },      /* print out messages. */
+#ifdef MS_SILENT
+  { "quiet",	0, 0, MS_SILENT    },	/* be quiet  */
+  { "loud",	0, 1, MS_SILENT    },	/* print out messages. */
 #endif
-  { NULL,	0, 0		}
+#ifdef MS_MANDLOCK
+  { "mand",	0, 0, MS_MANDLOCK },	/* Allow mandatory locks on this FS */
+  { "nomand",	0, 1, MS_MANDLOCK },	/* Forbid mandatory locks on this FS */
+#endif
+  { "loop",	1, 0, MS_LOOP	},	/* use a loop device */
+  { NULL,	0, 0, 0		}
 };
 
+char *opt_loopdev, *opt_vfstype, *opt_offset, *opt_encryption;
+
+struct string_opt_map {
+  char *tag;
+  int skip;
+  char **valptr;
+} string_opt_map[] = {
+  { "loop=",	0, &opt_loopdev },
+  { "vfs=",	1, &opt_vfstype },
+  { "offset=",	0, &opt_offset },
+  { "encryption=", 0, &opt_encryption },
+  { NULL, 0, NULL }
+};
+
+static void
+clear_string_opts(void) {
+  struct string_opt_map *m;
+
+  for (m = &string_opt_map[0]; m->tag; m++)
+    *(m->valptr) = NULL;
+}
+
+static int
+parse_string_opt(char *s) {
+  struct string_opt_map *m;
+  int lth;
+
+  for (m = &string_opt_map[0]; m->tag; m++) {
+    lth = strlen(m->tag);
+    if (!strncmp(s, m->tag, lth)) {
+      *(m->valptr) = xstrdup(s + lth);
+      return 1;
+    }
+  }
+  return 0;
+}
+
 int mount_quiet=0;
 
 /* Report on a single mount.  */
@@ -147,7 +198,7 @@ print_all (string_list types)
 
 
 /* Look for OPT in opt_map table and return mask value.  If OPT isn't found,
-   tack it onto extra_opts.  */
+   tack it onto extra_opts (which is non-NULL).  */
 static inline void
 parse_opt (const char *opt, int *mask, char *extra_opts)
 {
@@ -185,15 +236,16 @@ parse_opts (char *opts, int *flags, char **extra_opts)
   *flags = 0;
   *extra_opts = NULL;
 
+  clear_string_opts();
+
   if (opts != NULL)
     {
       *extra_opts = xmalloc (strlen (opts) + 1); 
       **extra_opts = '\0';
 
-      for (opt = strtok (opts, ",");
-	   opt != NULL;
-	   opt = strtok (NULL, ","))
-	parse_opt (opt, flags, *extra_opts);
+      for (opt = strtok (opts, ","); opt; opt = strtok (NULL, ","))
+	if (!parse_string_opt (opt))
+	  parse_opt (opt, flags, *extra_opts);
     }
 
   if (readonly)
@@ -207,13 +259,13 @@ static char *
 fix_opts_string (int flags, char *extra_opts)
 {
   const struct opt_map *om;
+  const struct string_opt_map *m;
   char *new_opts;
   char *tmp;
 
   new_opts = (flags & MS_RDONLY) ? "ro" : "rw";
-  for (om = opt_map; om->opt != NULL; om++)
-    {
-      if (om->mask & MS_RDONLY)
+  for (om = opt_map; om->opt != NULL; om++) {
+      if (om->skip)
 	continue;
       if (om->inv || !om->mask || (flags & om->mask) != om->mask)
 	continue;
@@ -221,17 +273,22 @@ fix_opts_string (int flags, char *extra_opts)
       sprintf(tmp, "%s,%s", new_opts, om->opt);
       new_opts = tmp;
       flags &= ~om->mask;
-    }
-  if (extra_opts && *extra_opts)
-    {
+  }
+  for (m = &string_opt_map[0]; m->tag; m++) {
+      if (!m->skip && *(m->valptr)) {
+	  tmp = xmalloc(strlen(new_opts) + strlen(m->tag) + strlen(*(m->valptr)) + 2);
+	  sprintf(tmp, "%s,%s%s", new_opts, m->tag, *(m->valptr));
+	  new_opts = tmp;
+      }
+  }
+  if (extra_opts && *extra_opts) {
       tmp = xmalloc(strlen(new_opts) + strlen(extra_opts) + 2);
       sprintf(tmp, "%s,%s", new_opts, extra_opts);
       new_opts = tmp;
-    }
+  }
   return new_opts;
 }
 
-
 /*
     char *fstype(const char *device);
 
@@ -242,6 +299,8 @@ fix_opts_string (int flags, char *extra_opts)
     for mount(8) by Mike Grupenhoff <kashmir@umiacs.umd.edu>.
     Read the superblock only once - aeb
     Added a test for iso9660 - aeb
+    Added a test for high sierra (iso9660) - quinlan@bucknell.edu
+    Corrected the test for xiafs - aeb
 
     Currently supports: minix, ext, ext2, xiafs, iso9660
 */
@@ -261,65 +320,63 @@ static char *
 fstype(const char *device)
 {
     int fd;
-    char ifs_magic[8];
+    char *type = NULL;
     union {
 	struct minix_super_block ms;
 	struct ext_super_block es;
 	struct ext2_super_block e2s;
-	struct xiafs_super_block xfs;
     } sb;
+    struct xiafs_super_block xiasb;
+    union {
+	struct iso_volume_descriptor iso;
+	struct hs_volume_descriptor hs;
+    } isosb;
     struct stat statbuf;
 
     /* opening and reading an arbitrary unknown path can have
        undesired side effects - first check that `device' refers
        to a block device */
-    if (stat (device, &statbuf))
-       error("mount: %s does not exist", device);
-    if (!S_ISBLK(statbuf.st_mode))
-       error("mount: %s is not a block device", device);
+    if (stat (device, &statbuf) || !S_ISBLK(statbuf.st_mode))
+      return 0;
 
     fd = open(device, O_RDONLY);
-    if (fd < 0
-	|| lseek(fd, BLOCK_SIZE, SEEK_SET) < 0
+    if (fd < 0)
+      return 0;
+
+    if (lseek(fd, BLOCK_SIZE, SEEK_SET) < 0
 	|| read(fd, (char *) &sb, sizeof(sb)) < 0) {
 	perror(device);
+	close(fd);
 	return 0;
     }
 
     if (sb.ms.s_magic == MINIX_SUPER_MAGIC
-	|| sb.ms.s_magic == MINIX_SUPER_MAGIC2) {
-	close (fd);
-	return("minix");
-    }
+	|| sb.ms.s_magic == MINIX_SUPER_MAGIC2)
+      type = "minix";
 
-    if (sb.es.s_magic == EXT_SUPER_MAGIC) {
-	close (fd);
-	return("ext");
-    }
+    else if (sb.es.s_magic == EXT_SUPER_MAGIC)
+      type = "ext";
 
 #ifndef EXT2_PRE_02B_MAGIC
 #define EXT2_PRE_02B_MAGIC    0xEF51
 #endif
-    if (sb.e2s.s_magic == EXT2_SUPER_MAGIC
-	|| sb.e2s.s_magic == EXT2_PRE_02B_MAGIC) {
-        close (fd);
-	return("ext2");
-    }
+    else if (sb.e2s.s_magic == EXT2_SUPER_MAGIC
+	     || sb.e2s.s_magic == EXT2_PRE_02B_MAGIC)
+      type = "ext2";
 
-    if (sb.xfs.s_magic == _XIAFS_SUPER_MAGIC) {
-        close (fd);
-	return("xiafs");
-    }
+    else if (lseek(fd, 0, SEEK_SET) == 0
+	&& read(fd, (char *) &xiasb, sizeof(xiasb)) == sizeof(xiasb)
+	&& xiasb.s_magic == _XIAFS_SUPER_MAGIC)
+      type = "xiafs";
 
-    if (lseek (fd, 0100000, SEEK_SET) != -1
-        && read (fd, ifs_magic, 8) == 8
-        && !strncmp(ifs_magic, "\001CD001\001", 8)) { /* ECMA 119 */
-        close (fd);
-        return("iso9660");
-    }
+    else if (lseek(fd, 0x8000, SEEK_SET) == 0x8000
+        && read(fd, (char *) &isosb, sizeof(isosb)) == sizeof(isosb)
+        && (strncmp(isosb.iso.id, ISO_STANDARD_ID, sizeof(isosb.iso.id)) == 0
+	    || strncmp(isosb.hs.id, HS_STANDARD_ID, sizeof(isosb.hs.id)) == 0))
+      type = "iso9660";
 
     close (fd);
-    return(0);
+    return(type);
 }
 
 FILE *procfs;
@@ -384,7 +441,7 @@ static int
 try_mount5 (char *spec, char *node, char **type, int flags, char *mount_opts) {
    char *fsname;
    
-   if (strcasecmp (*type, "auto") == 0)
+   if (*type && strcasecmp (*type, "auto") == 0)
       *type = NULL;
 
    if (!*type) {
@@ -426,21 +483,41 @@ try_mount5 (char *spec, char *node, char **type, int flags, char *mount_opts) {
 
 
 static int
-mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
+mount_one (char *spec0, char *node0, char *type0, char *opts0,
+	   int freq, int pass)
 {
   struct mntent mnt;
   int mnt_err;
   int flags;
-  char *extra_opts;
-  char *mount_opts;
+  char *extra_opts;		/* written in mtab */
+  char *mount_opts;		/* actually used on system call */
   static int added_ro = 0;
-  int loop=0;
+  int loop, looptype, offset;
+  char *spec, *node, *type, *opts, *loopdev, *loopfile;
 
-  if (type == NULL)
-    {
-      if (strchr (spec, ':') != NULL)
+  spec = xstrdup(spec0);
+  node = xstrdup(node0);
+  type = xstrdup(type0);
+  opts = xstrdup(opts0);
+
+/*
+ * There are two special cases: nfs mounts and loop mounts.
+ * In the case of nfs mounts spec is probably of the form machine:path.
+ * In the case of a loop mount, either type is of the form lo@/dev/loop5
+ * or the option "-o loop=/dev/loop5" or just "-o loop" is given, or
+ * mount just has to figure things out for itself from the fact that
+ * spec is not a block device. We do not test for a block device
+ * immediately: maybe later other types of mountable objects will occur.
+ */
+
+  if (type == NULL) {
+      if (strchr (spec, ':') != NULL) {
 	type = "nfs";
-    }
+	if (verbose)
+	  printf("mount: no type was given - "
+		 "I'll assume nfs because of the colon\n");
+      }
+  }
 
   parse_opts (xstrdup (opts), &flags, &extra_opts);
 
@@ -458,17 +535,51 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
 
   mount_opts = extra_opts;
 
-  if (!fake && type && strncmp("lo@", type, 3)==0) {
-    extern int lomount (char *, char *, char *, char **, 
-			int *, char **, char **);
-    char *dev=type+3;
+  /*
+   * A loop mount?
+   * Only test for explicitly specified loop here,
+   * and try implicit loop if the mount fails.
+   */
+  loopdev = opt_loopdev;
 
-    loop=1;
-    if (lomount (spec, node, dev, &type,
-		 &flags, &opts, &mount_opts) != 0)
-      return 1;
-    spec=dev;
-    mount_opts=NULL;
+  looptype = (type && strncmp("lo@", type, 3) == 0);
+  if (looptype) {
+    if (loopdev)
+      error("mount: loop device specified twice");
+    loopdev = type+3;
+    type = opt_vfstype;
+  }
+  else if (opt_vfstype) {
+    if (type)
+      error("mount: type specified twice\n");
+    else
+      type = opt_vfstype;
+  }
+
+  loop = ((flags & MS_LOOP) || loopdev || opt_offset || opt_encryption);
+  loopfile = spec;
+
+  if (loop) {
+    flags |= MS_LOOP;
+    if (fake) {
+      if (verbose)
+	printf("mount: skipping the setup of a loop device\n");
+    } else {
+      int loopro = (flags & MS_RDONLY);
+
+      if (!loopdev || !*loopdev)
+	loopdev = find_unused_loop_device();
+      if (!loopdev)
+	return 1;
+      if (verbose)
+	printf("mount: going to use the loop device %s\n", loopdev);
+      offset = opt_offset ? strtoul(opt_offset, NULL, 0) : 0;
+      if (set_loop (loopdev, loopfile, offset, opt_encryption, &loopro))
+	return 1;
+      spec = loopdev;
+      if (loopro)
+	flags |= MS_RDONLY;
+    }
   }
 
   if (!fake && type && streq (type, "nfs"))
@@ -483,44 +594,44 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
 
   if (fake
       || (try_mount5 (spec, node, &type, flags & ~MS_NOSYS, mount_opts)) == 0)
-    /* Mount succeeded, write mtab entry.  */
+    /* Mount succeeded, report this (if verbose) and write mtab entry.  */
     {
-      if (!nomtab)
-	{
-	  mnt.mnt_fsname = canonicalize (spec);
-	  mnt.mnt_dir = canonicalize (node);
-	  mnt.mnt_type = loop ? "loop" : type;
-	  mnt.mnt_opts = fix_opts_string (flags & ~MS_NOMTAB, 
-					  loop?opts:extra_opts);
-	  mnt.mnt_freq = freq;
-	  mnt.mnt_passno = pass;
-      
-	  /* We get chatty now rather than after the update to mtab since the
-	     mount succeeded, even if the write to /etc/mtab should fail.  */
-	  if (verbose)
-	    print_one (&mnt);
+      if (loop)
+	  opt_loopdev = loopdev;
 
-	  if (flags & MS_REMOUNT)
-	    {
-	      close_mtab ();
-	      update_mtab (mnt.mnt_dir, &mnt);
-	      open_mtab ("a+");
-	    }
-	  else
+      mnt.mnt_fsname = canonicalize (loop ? loopfile : spec);
+      mnt.mnt_dir = canonicalize (node);
+      mnt.mnt_type = type ? type : "unknown";
+      mnt.mnt_opts = fix_opts_string (flags & ~MS_NOMTAB, extra_opts);
+      mnt.mnt_freq = freq;
+      mnt.mnt_passno = pass;
+      
+      /* We get chatty now rather than after the update to mtab since the
+	 mount succeeded, even if the write to /etc/mtab should fail.  */
+      if (verbose)
+	   print_one (&mnt);
+
+      if (!nomtab) {
+	  if (flags & MS_REMOUNT) {
+	    close_mtab ();
+	    update_mtab (mnt.mnt_dir, &mnt);
+	    open_mtab ("a+");
+	  } else {
 	    if ((addmntent (F_mtab, &mnt)) == 1)
 	      die (1, "mount: error writing %s: %s",
 		   MOUNTED, strerror (errno));
-	}
+	  }
+      }
 
       block_signals (SIG_UNBLOCK);
       return 0;
     }
 
+  mnt_err = errno;
+
   if (loop)
 	del_loop(spec);
 
-  mnt_err = errno; /* work around for errno bug in sigprocmask */
-
   block_signals (SIG_UNBLOCK);
 
   /* Mount failed, complain, but don't die.  */
@@ -537,8 +648,12 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
 	error ("mount: must be superuser to use mount");
       break;
     case EBUSY:
-      error ("mount: %s already mounted or %s busy", spec, node);
-      already (spec, node);
+      if (flags & MS_REMOUNT) {
+	error ("mount: %s is busy", node);
+      } else {
+	error ("mount: %s already mounted or %s busy", spec, node);
+	already (spec, node);
+      }
       break;
     case ENOENT:
       { struct stat statbuf;
@@ -558,22 +673,67 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
     case ENOTDIR:
       error ("mount: mount point %s is not a directory", node); break;
     case EINVAL:
-      error ("mount: wrong fs type or bad superblock on %s", spec); break;
+    { int fd, size;
+
+      error ("mount: wrong fs type, bad option, bad superblock on %s,\n"
+	     "       or too many mounted file systems",
+	     spec);
+
+      if ((fd = open(spec, O_RDONLY)) >= 0) {
+	if(ioctl(fd, BLKGETSIZE, &size) == 0 && size <= 2)
+	  error ("       (aren't you trying to mount an extended partition,\n"
+		 "       instead of some logical partition inside?)\n");
+	close(fd);
+      }
+
+      break;
+    }
     case EMFILE:
       error ("mount table full"); break;
     case EIO:
       error ("mount: %s: can't read superblock", spec); break;
     case ENODEV:
-      if (is_in_proc(type))
+      if (is_in_proc(type) || !strcmp(type, "guess"))
         error("mount: %s has wrong major or minor number", spec);
-      else if (procfs)
+      else if (procfs) {
+	char *lowtype, *p;
+	int u;
+
 	error ("mount: fs type %s not supported by kernel", type);
-      else
+
+	/* maybe this loser asked for FAT or ISO9660 or isofs */
+	lowtype = xstrdup(type);
+	u = 0;
+	for(p=lowtype; *p; p++) {
+	  if(tolower(*p) != *p) {
+	    *p = tolower(*p);
+	    u++;
+	  }
+	}
+	if (u && is_in_proc(lowtype))
+	  error ("mount: probably you meant %s", lowtype);
+	else if (!strncmp(lowtype, "iso", 3) && is_in_proc("iso9660"))
+	  error ("mount: maybe you meant iso9660 ?");
+	free(lowtype);
+      } else
 	error ("mount: %s has wrong device number or fs type %s not supported",
 	       spec, type);
       break;
     case ENOTBLK:
-      error ("mount: %s is not a block device", spec); break;
+      { struct stat statbuf;
+
+	if (stat (spec, &statbuf)) /* strange ... */
+	  error ("mount: %s is not a block device, and stat fails?", spec);
+	else if (S_ISBLK(statbuf.st_mode))
+	  error ("mount: the kernel does not recognize %s as a block device\n"
+		 "       (maybe `insmod driver'?)", spec);
+	else if (S_ISREG(statbuf.st_mode))
+	  error ("mount: %s is not a block device (maybe try `-o loop'?)",
+		 spec);
+	else
+	  error ("mount: %s is not a block device", spec);
+      }
+      break;
     case ENXIO:
       error ("mount: %s is not a valid block device", spec); break;
     case EACCES:  /* pre-linux 1.1.38 */
@@ -584,6 +744,10 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
           break;
       } else {
          added_ro = 1;
+	 if (loop) {
+	     opts = opts0;
+	     type = type0;
+	 }
          if (opts) {
              opts = realloc(xstrdup(opts), strlen(opts)+4);
              strcat(opts, ",ro");
@@ -591,9 +755,9 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
              opts = "ro";
 	 if (type && !strcmp(type, "guess"))
 	     type = 0;
-         error ("mount: block device %s is write-protected, "
-		"mounting read-only", spec);
-	 return mount_one (spec, node, type, opts, freq, pass);
+         error ("mount: %s%s is write-protected, mounting read-only",
+		loop ? "" : "block device ", spec0);
+	 return mount_one (spec0, node0, type, opts, freq, pass);
       }
       break;
     default:
@@ -641,7 +805,8 @@ mount_all (string_list types)
       }
 
   status = 0;
-  if (!setfsent()) return 1;
+  if (!setfsent())
+    return 1;
   while ((fstab = getfsent ()) != NULL)
     if (matching_type (fstab->mnt_type, types)
 	 && !streq (fstab->mnt_dir, "/")
@@ -675,8 +840,7 @@ create_mtab (void)
   /* Find the root entry by looking it up in fstab, which might be wrong.
      We could statfs "/" followed by a slew of stats on /dev/ but then
      we'd have to unparse the mount options as well....  */
-  if ((fstab = getfsfile ("/")) || (fstab = getfsfile ("root")))
-    {
+  if ((fstab = getfsfile ("/")) || (fstab = getfsfile ("root"))) {
       parse_opts (xstrdup (fstab->mnt_opts), &flags, &extra_opts);
       mnt = *fstab;
       mnt.mnt_fsname = canonicalize (fstab->mnt_fsname);
@@ -685,10 +849,8 @@ create_mtab (void)
 
       if (addmntent (F_mtab, &mnt) == 1)
 	die (1, "mount: error writing %s: %s", MOUNTED, strerror (errno));
-    }
-  if (fchmod (fileno (F_mtab), S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH) < 0)
-    die (1, "mount: error changing mode of %s: %s", MOUNTED, strerror (errno));
-  endmntent (F_mtab);
+  }
+  close_mtab();
 }
 
 extern char version[];
@@ -725,9 +887,8 @@ usage (FILE *fp, int n)
 }
 
 int
-main (int argc, char *argv[])
-{
-  int c;
+main (int argc, char *argv[]) {
+  int c, err;
   char *options = NULL;
   string_list types = NULL;
   struct mntent *fs;
@@ -736,8 +897,7 @@ main (int argc, char *argv[])
   struct stat statbuf;
 
   while ((c = getopt_long (argc, argv, "afhnrvVwt:o:", longopts, NULL)) != EOF)
-    switch (c)
-      {
+    switch (c) {
       case 'a':			/* mount everything in fstab */
 	++all;
 	break;
@@ -750,11 +910,17 @@ main (int argc, char *argv[])
       case 'n':			/* mount without writing in /etc/mtab */
 	++nomtab;
 	break;
+      case 'o':			/* specify mount options */
+	options = optarg;
+	break;
       case 'r':			/* mount readonly */
 	++readonly;
 	readwrite = 0;
 	break;
-      case 'v':			/* be chatty */
+      case 't':			/* specify file system types */
+	types = parse_list (optarg);
+	break;
+      case 'v':			/* be chatty - very chatty if repeated */
 	++verbose;
 	break;
       case 'V':			/* version */
@@ -764,48 +930,40 @@ main (int argc, char *argv[])
 	++readwrite;
 	readonly = 0;
 	break;
-      case 't':			/* specify file system types */
-	types = parse_list (optarg);
-	break;
-      case 'o':			/* specify mount options */
-	options = optarg;
-	break;
       case 0:
 	break;
       case '?':
       default:
 	usage (stderr, 1);
 	break;
-      }
+    }
 
   argc -= optind;
   argv += optind;
 
-  if (argc == 0)
-    {
+  if (argc == 0) {
       if (options)
 	usage (stderr, 1);
       if (!all)
 	return print_all (types);
-    }
+  }
 
-  if (getuid () != geteuid ())
-    {
+  if (getuid () != geteuid ()) {
       suid = 1;
       if (types || options || readwrite || nomtab || all || fake || argc != 1)
 	die (2, "mount: only root can do that");
-    }
+  }
 
-  if (!nomtab)
-    {
+  err = stat(MOUNTED, &statbuf);
+  if (!nomtab) {
       lock_mtab ();
-      if (stat(MOUNTED, &statbuf) < 0)
-	create_mtab ();
+      if (err)
+	  create_mtab ();
       open_mtab ("a+");
-    }
-  else if (stat(MOUNTED, &statbuf) >= 0)
-    open_mtab ("r");
-
+  } else {
+      if (!err)
+	  open_mtab ("r");
+  }
 
   switch (argc)
     {
