@@ -14,6 +14,9 @@
  *
  * Wed Feb  8 12:51:48 1995, biro@yggdrasil.com (Ross Biro): allow all port
  * numbers to be specified on the command line.
+ *
+ * Fri, 8 Mar 1996 18:01:39, Swen Thuemmler <swen@uni-paderborn.de>:
+ * Omit the call to connect() for Linux version 1.3.11 or later.
  */
 
 /*
@@ -26,14 +29,14 @@
 #include <rpc/pmap_clnt.h>
 #include <sys/socket.h>
 #include <sys/time.h>
+#include <sys/utsname.h>
 #include <string.h>
 #include <netdb.h>
 #include <arpa/inet.h>
 #include <errno.h>
 
 #include "sundries.h"
-
-#include "mount.h"
+#include "nfsmount.h"
 
 #include <linux/fs.h>
 #include <linux/nfs.h>
@@ -52,6 +55,19 @@ static char *strndup (char *str, int n) {
 
 static char *nfs_strerror(int stat);
 
+static int
+linux_version_code(void) {
+	struct utsname my_utsname;
+	int release = 0;
+  
+	if (0 == uname(&my_utsname)) {
+	  release = 65536 * atoi(strtok(my_utsname.release, "."))
+	           +  256 * atoi(strtok(NULL, "."))
+		   +        atoi(strtok(NULL, "."));
+	}
+	return release;
+}
+
 int nfsmount(const char *spec, const char *node, int *flags,
 	     char **extra_opts, char **mount_opts)
 {
@@ -92,6 +108,11 @@ int nfsmount(const char *spec, const char *node, int *flags,
 
 	msock = fsock = -1;
 	mclient = NULL;
+	if (strlen(spec) >= sizeof(hostdir)) {
+		fprintf(stderr, "mount: "
+			"excessively long host:dir argument\n");
+		goto fail;
+	}
 	strcpy(hostdir, spec);
 	if ((s = (strchr(hostdir, ':')))) {
 		hostname = hostdir;
@@ -121,13 +142,18 @@ int nfsmount(const char *spec, const char *node, int *flags,
 
 	/* add IP address to mtab options for use when unmounting */
 
+	s = inet_ntoa(server_addr.sin_addr);
 	old_opts = *extra_opts;
 	if (!old_opts)
 		old_opts = "";
+	if (strlen(old_opts) + strlen(s) + 10 >= sizeof(new_opts)) {
+		fprintf(stderr, "mount: "
+			"excessively long option argument\n");
+		goto fail;
+	}
 	sprintf(new_opts, "%s%saddr=%s",
-		old_opts, *old_opts ? "," : "",
-		inet_ntoa(server_addr.sin_addr));
-	*extra_opts = strdup(new_opts);
+		old_opts, *old_opts ? "," : "", s);
+	*extra_opts = xstrdup(new_opts);
 
 	/* set default options */
 
@@ -336,8 +362,8 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	/* try to mount hostname:dirname */
 
 	clnt_stat = clnt_call(mclient, MOUNTPROC_MNT,
-		xdr_dirpath, &dirname,
-		xdr_fhstatus, &status,
+		(xdrproc_t) xdr_dirpath, &dirname,
+		(xdrproc_t) xdr_fhstatus, &status,
 		total_timeout);
 	if (clnt_stat != RPC_SUCCESS) {
 		clnt_perror(mclient, "rpc mount");
@@ -387,8 +413,14 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	printf("using port %d for nfs deamon\n", port);
 #endif
 	server_addr.sin_port = htons(port);
-	if (connect(fsock, (struct sockaddr *) &server_addr,
-	    sizeof (server_addr)) < 0) {
+	 /*
+	  * connect() the socket for kernels 1.3.10 and below only,
+	  * to avoid problems with multihomed hosts.
+	  * --Swen
+	  */
+	if (linux_version_code() <= 66314
+	    && connect(fsock, (struct sockaddr *) &server_addr,
+		       sizeof (server_addr)) < 0) {
 		perror("nfs connect");
 		goto fail;
 	}
@@ -418,7 +450,8 @@ fail:
 	}
 	if (fsock != -1)
 		close(fsock);
-	return 1;}
+	return 1;
+}
 	
 
 /*
