@@ -1,9 +1,17 @@
 /* Original author unknown, but may be "krishna balasub@cis.ohio-state.edu"
    Modified Sat Oct  9 10:55:28 1993 for 0.99.13 */
 
-/* Patches from Mike Jagdis (jaggy@purplet.demon.co.uk) applied Wed Feb 8
-12:12:21 1995 by faith@cs.unc.edu to print numeric uids if no passwd file
-entry. */
+/* 
+
+  Patches from Mike Jagdis (jaggy@purplet.demon.co.uk) applied Wed Feb
+  8 12:12:21 1995 by faith@cs.unc.edu to print numeric uids if no
+  passwd file entry.
+
+  Patch from arnolds@ifns.de (Heinz-Ado Arnolds) applied Mon Jul 1
+  19:30:41 1996 by janl@math.uio.no to add code missing in case PID:
+  clauses.
+
+*/
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -412,6 +420,8 @@ void do_msg (char format)
 		break;
 
 	case PID:
+ 		printf ("------ Message Queues PIDs --------\n");
+ 		printf ("%-10s%-10s%-10s%-10s\n","msqid","owner","lspid","lrpid");
 		break;
 
 	default:
@@ -442,8 +452,14 @@ void do_msg (char format)
 			msgque.msg_ctime ? ctime(&msgque.msg_ctime) + 4 : "Not set");
 			break;
 		case PID:
-			break;
-			
+ 			if (pw)
+ 				printf ("%-8d%-10.10s", msqid, pw->pw_name);
+ 			else
+ 				printf ("%-8d%-10d", msqid, ipcp->uid);
+ 			printf ("  %5d     %5d\n",
+ 			msgque.msg_lspid, msgque.msg_lrpid);
+  			break;
+
 		default:
 			if (pw)
 				printf ("%-10d%-10.10s", msqid, pw->pw_name);
