@@ -32,6 +32,8 @@ static char sccsid[] = "@(#)more.c	5.19 (Berkeley) 6/29/88";
 **
 **	modified by Geoff Peck, UCB to add underlining, single spacing
 **	modified by John Foderaro, UCB to add -c and MORE environment variable
+**	modified by Erik Troan <ewt@redhat.com> to be more posix and so compile
+**	  on linux/axp.
 */
 
 #include <stdio.h>
@@ -42,6 +44,7 @@ static char sccsid[] = "@(#)more.c	5.19 (Berkeley) 6/29/88";
 #include <errno.h>
 #include <termios.h>
 #include <setjmp.h>
+#include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/file.h>
 #include <a.out.h>
@@ -57,7 +60,7 @@ static char sccsid[] = "@(#)more.c	5.19 (Berkeley) 6/29/88";
 #define Getc(f)		(++file_pos, getc(f))
 #define Ungetc(c,f)	(--file_pos, ungetc(c,f))
 
-#define stty(fd,argp)	ioctl(fd,TCSETAF,argp)
+#define stty(fd,argp)	ioctl(fd,TCSETSF,argp)
 
 /* some function declarations */
 void initterm();
@@ -74,7 +77,7 @@ void cleareol();
 void clreos();
 int  pr(char *s1);
 void reset_tty();
-int  getline(register FILE *f, int *length);
+int  get_line(register FILE *f, int *length);
 void prbuf (register char *s, register int n);
 
 #define TBUFSIZ	1024
@@ -84,7 +87,7 @@ void prbuf (register char *s, register int n);
 #define ESC	'\033'
 #define QUIT	'\034'
 
-struct termio	otty, save_tty;
+struct termios	otty, savetty;
 long		file_pos, file_size;
 int		fnum, no_intty, no_tty, slow_tty;
 int		dum_opt, dlines;
@@ -456,7 +459,7 @@ void screen (register FILE *f, register int num_lines)
 
     for (;;) {
 	while (num_lines > 0 && !Pause) {
-	    if ((nchars = getline (f, &length)) == EOF)
+	    if ((nchars = get_line (f, &length)) == EOF)
 	    {
 		if (clreol)
 		    clreos();
@@ -753,7 +756,7 @@ char *filename;
 ** Get a logical line
 */
 
-int getline(register FILE *f, int *length)
+int get_line(register FILE *f, int *length)
 {
     register int	c;
     register char	*p;
@@ -802,16 +805,8 @@ int getline(register FILE *f, int *length)
 		column = 1 + (column | 7);
 	else if (c == '\b' && column > 0)
 	    column--;
-	else if (c == '\r') {	/* fixes bug with "\r\n" at end of line */
-	    int next = Getc(f);
-            if (next == '\n') {
-              *p--;
-	      Currline++;
-	      break;
-            }
-	    Ungetc(next,f);
+	else if (c == '\r')
 	    column = 0;
-        }
 	else if (c == '\f' && stop_opt) {
 		p[-1] = '^';
 		*p++ = 'L';
@@ -1399,6 +1394,9 @@ va_dcl
 	int id;
 	int n;
 	va_list argp;
+	char * arg;
+ 	char ** args;
+	int argcount;
 
 	fflush (stdout);
 	reset_tty ();
@@ -1409,8 +1407,40 @@ va_dcl
 		close(0);
 		open("/dev/tty", 0);
 	    }
+
+#if 0
+	    /* I don't feel too safe about removing this... - janl */
 	    va_start(argp);
-	    execv (cmd, argp);
+
+	    arg = va_arg(argp, char *);
+	    argcount = 0;
+	    while (arg) {
+		argcount++;
+	        arg = va_arg(argp, char *);
+	    }
+	    va_end(argp);	/* balance {}'s for some UNIX's */
+
+	    args = alloca(sizeof(char *) * (argcount + 1));
+	    args[argcount] = NULL;
+#endif
+	    
+	    va_start(argp);
+#if 0
+	    arg = va_arg(argp, char *);
+	    argcount = 0;
+	    while (arg) {
+		args[argcount] = arg;
+		argcount++;
+	        arg = va_arg(argp, char *);
+	    }
+	    va_end(argp);	/* balance {}'s for some UNIX's */
+#endif
+	
+#ifdef __alpha__ 
+	    execv (cmd, &argp);
+#else
+	    execv (cmd, args);
+#endif
 	    write (2, "exec failed\n", 12);
 	    exit (1);
 	    va_end(argp);	/* balance {}'s for some UNIX's */
@@ -1494,7 +1524,7 @@ void initterm()
     struct winsize win;
 
 retry:
-    no_tty = ioctl(fileno(stdout), TCGETA, &otty);
+    no_tty = ioctl(fileno(stdout), TCGETS, &otty);
     if (!no_tty) {	
 	docrterase = (otty.c_cc[VERASE] != 255);
 	docrtkill =  (otty.c_cc[VKILL] != 255);
@@ -1592,9 +1622,9 @@ retry:
 	if ((shell = getenv("SHELL")) == NULL)
 	    shell = "/bin/sh";
     }
-    no_intty = ioctl(fileno(stdin), TCGETA, &otty);
-    ioctl(fileno(stderr), TCGETA, &otty);
-    save_tty = otty;
+    no_intty = ioctl(fileno(stdin), TCGETS, &otty);
+    ioctl(fileno(stderr), TCGETS, &otty);
+    savetty = otty;
     slow_tty = (otty.c_cflag & CBAUD) < B1200;
     hardtabs = (otty.c_oflag & TABDLY) != XTABS;
     if (!no_tty) 
@@ -1800,7 +1830,7 @@ void reset_tty ()
 	pstate = 0;
     }
     otty.c_lflag |= ICANON|ECHO;
-    stty(fileno(stderr), &save_tty);
+    stty(fileno(stderr), &savetty);
 }
 
 rdline (f)
